Imports Microsoft.VisualBasic
Imports System
Imports System.Data
Imports System.Data.SqlClient

Namespace ProductList

    Public Class Categories

#Region "Variables"
        Private _CategoryID As Int32
        Private _LinkName As String
        Private _Description As String
        Private _HeadCategoryID As Int32
#End Region

#Region "Constructors"
        Public Sub New()
            Me.CategoryID = 0
            Me.LinkName = String.Empty
            Me.Description = String.Empty
            Me.HeadCategoryID = 0
        End Sub
        Public Sub New(ByVal pLinkName As String, ByVal pDescription As String, ByVal pHeadCategoryID As Int32)
            Me.LinkName = pLinkName
            Me.Description = pDescription
            Me.HeadCategoryID = pHeadCategoryID
        End Sub
        Public Sub New(ByVal pCategoryID As Int32, ByVal pLinkName As String, ByVal pDescription As String, _
        ByVal pHeadCategoryID As Int32)
            Me.CategoryID = pCategoryID
            Me.LinkName = pLinkName
            Me.Description = pDescription
            Me.HeadCategoryID = pHeadCategoryID
        End Sub
        Protected Sub New(ByVal reader As SqlDataReader)
            Me.CategoryID = Convert.ToInt32(reader("CategoryID"))
            Me.LinkName = reader("LinkName").ToString()
            Me.Description = reader("Description").ToString()
            Me.HeadCategoryID = Convert.ToInt32(reader("HeadCategoryID"))
        End Sub
#End Region

#Region "Properties"

        Public Property CategoryID() As Int32
            Get
                Return _CategoryID
            End Get
            Set(ByVal value As Int32)
                _CategoryID = value
            End Set
        End Property

        Public Property LinkName() As String
            Get
                Return _LinkName
            End Get
            Set(ByVal value As String)
                _LinkName = value
            End Set
        End Property

        Public Property Description() As String
            Get
                Return _Description
            End Get
            Set(ByVal value As String)
                _Description = value
            End Set
        End Property

        Public Property HeadCategoryID() As Int32
            Get
                Return _HeadCategoryID
            End Get
            Set(ByVal value As Int32)
                _HeadCategoryID = value
            End Set
        End Property

#End Region

#Region "Methods"

        ''' <summary>
        ''' Insert one category row with LinkName, Description and HeadCategoryID based on the values for the 
        ''' variables LinkName, Description and HeadCategoryID. You need to set values for these properties before calling
        ''' this method, CategoryId is a identity column that increments with 1.
        ''' </summary>
        Public Overloads Sub InsertOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "INSERT INTO Categories (LinkName, Description, HeadCategoryID) VALUES" & _
                "(@LinkName, @Description, @HeadCategoryID)"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(Sql, cn)
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50).Value = Me.LinkName
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200).Value = Me.Description
                    cmd.Parameters.Add("@HeadCategoryID", SqlDbType.Int).Value = Me.HeadCategoryID

                    cn.Open()
                    ' Execute the INSERT statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Insert one category row by using the parameter values supplied when this method is called, 
        ''' the CategoryId column is a identity that increments with 1.
        ''' </summary>
        Public Overloads Sub InsertOneRow(ByVal pLinkName As String, ByVal pDescription As String, _
        ByVal pHeadCategoryID As Int32)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "INSERT INTO Categories (LinkName, Description, HeadCategoryID) VALUES" & _
                "(@LinkName, @Description, @HeadCategoryID)"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50).Value = pLinkName
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200).Value = pDescription
                    cmd.Parameters.Add("@HeadCategoryID", SqlDbType.Int).Value = pHeadCategoryID

                    cn.Open()
                    ' Execute the INSERT statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Update one category row where the CategoryId equals the CategoryId variable in this class, you need to set values
        ''' for CategoryId, LinkName, Description and HeadCategoryID with a constructor or with properties before using this method.
        ''' </summary>
        Public Overloads Sub UpdateOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "UPDATE Categories SET LinkName = @LinkName, Description = @Description, HeadCategoryID = @HeadCategoryID WHERE CategoryID = @Id"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = Me.CategoryID
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50).Value = Me.LinkName
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200).Value = Me.Description
                    cmd.Parameters.Add("@HeadCategoryID", SqlDbType.Int).Value = Me.HeadCategoryID

                    cn.Open()
                    ' Excecute the UPDATE statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Update one category row where the CategoryId equals the supplied parameter pCategoryId, you need to add 
        ''' four parameters to use this method.
        ''' </summary>
        Public Overloads Sub UpdateOneRow(ByVal pCategoryID As Int32, ByVal pLinkName As String, _
        ByVal pDescription As String, ByVal pHeadCategoryID As Int32)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "UPDATE Categories SET LinkName = @LinkName, Description = @Description, HeadCategoryID = @HeadCategoryID WHERE CategoryID = @Id"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = pCategoryID
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50).Value = pLinkName
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200).Value = pDescription
                    cmd.Parameters.Add("@HeadCategoryID", SqlDbType.Int).Value = pHeadCategoryID

                    cn.Open()
                    ' Excecute the UPDATE statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Select one category row where the CategoryId equals the value for the variable CategoryId in this class, which you
        ''' have to set with a constructor or a property before using this method. The selected row is used to set the
        ''' properties for CategoryId, LinkName, Description and HeadCategoryID so you have to get values from the properties to get the
        ''' selected data.
        ''' </summary>
        Public Overloads Sub SelectOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM Categories WHERE CategoryID = @CategoryID"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@CategoryID", SqlDbType.Int).Value = Me.CategoryID

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            Me.CategoryID = Convert.ToInt32(reader("CategoryID"))
                            Me.LinkName = reader("LinkName").ToString()
                            Me.Description = reader("Description").ToString()
                            Me.HeadCategoryID = Convert.ToInt32(reader("HeadCategoryID"))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Select one category row where CategoryId equals pCategoryID, you need supply a parameter value for CategoryId to use
        ''' this method. The selected row is used to set values for the properties: CategoryId, LinkName, Description and
        ''' HeadCategoryID. You need to get the values from these properties after calling this method.
        ''' </summary>
        Public Overloads Sub SelectOneRow(ByVal pCategoryID As Int32)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM Categories WHERE CategoryID = @CategoryID"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@CategoryID", SqlDbType.Int).Value = pCategoryID

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            Me.CategoryID = Convert.ToInt32(reader("CategoryID"))
                            Me.LinkName = reader("LinkName").ToString()
                            Me.Description = reader("Description").ToString()
                            Me.HeadCategoryID = Convert.ToInt32(reader("HeadCategoryID"))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub
        ''' <summary>
        ''' Select all category rows to an ArrayList of category objects (CategoryId, LinkName, Description, HeadCategoryID). An 
        ''' ArrayList of objects can be used directly as a datasource for a repeater control or other listcontrols.
        ''' </summary>
        Public Function SelectAllToArrayList() As ArrayList

            ' Declare AllToArrayList as a new ArrayList
            Dim AllToArrayList As New ArrayList

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM Categories"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader()
                        While reader.Read()
                            ' Add a category object to the ArrayList
                            AllToArrayList.Add(New Categories(reader))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a ArrayList
            Return AllToArrayList

        End Function

        ''' <summary>
        ''' Select all category rows to an ArrayList of category objects (CategoryId, LinkName, Description, HeadCategoryID) 
        ''' where the HeadCategoryID equals pHeadCategoryID value. An ArrayList of objects can be used directly as 
        ''' a datasource for a repeater control or other datalists.
        ''' </summary>
        Public Function SelectAllToArrayList(ByVal pHeadCategoryID As Int32) As ArrayList

            ' Declare AllToArrayList as a new ArrayList
            Dim AllToArrayList As New ArrayList

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM Categories WHERE HeadCategoryID = @HeadCategoryID"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@HeadCategoryID", SqlDbType.Int).Value = pHeadCategoryID

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader()
                        While reader.Read()
                            ' Add a category object to the ArrayList
                            AllToArrayList.Add(New Categories(reader))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a ArrayList
            Return AllToArrayList

        End Function

#End Region

    End Class

End Namespace
