Imports Microsoft.VisualBasic
Imports System
Imports System.Data
Imports System.Data.SqlClient

' The namespace is a way for us to classify classes into to groups, all of the classes that we create in this project
' are assigned to a "ProductList" namespace. If we want to use this class in a form we need to import the
' "ProductList" namespace.
Namespace ProductList

    ' This class is documented with XML tags, we use the <remarks> tags to describe a class or a region and use
    ' the <summary> to describe specific properties and methods. The content of a summary tag will be visible as
    ' a description for the property or method when someone are looking at it in user code. XML comments can be
    ' collapsed by the minus sign and expanded by the plus sign.

    ''' <remarks>
    ''' This class is used to insert, update and select headcategories in an MS SQL database table and to
    ''' create objects of one headcategory. This class offers flexibility to insert, update, and select headcategories
    ''' by using several overloaded constructors, properties and several overloaded methods.  
    ''' </remarks>
    Public Class HeadCategories

#Region "Variables"
        ''' <remarks>
        ''' The variables in this class are declared as private and thereby only reachable by the properties and
        ''' methods inside this class. These private variables is used to hold information about one headcategory.
        ''' When we set values for these variables and when we get values from these we variables we are always using
        ''' the properties in this class even when refereing to these variables inside this class. If we had declared
        ''' these variables as protected they would also be reachable by classes that inherits from this class but we
        ''' dont want that.  
        ''' </remarks>
        Private _HeadId As Int32
        Private _LinkName As String
        Private _Description As String
#End Region

#Region "Constructors"
        ''' <remarks>
        ''' This class has several overloaded constructors that can be used to when someone creates a new instance of
        ''' this class. We dont have to write the "Overloads" for constructors that should be overloaded, each 
        ''' overloaded constructor must have a unique order, number of or type of parameters. The "Me" keyword is
        ''' used to be specific that we are referencing a property or method in this class. We are always using
        ''' the properties to set values for private variables in the constructors.  
        ''' </remarks>
        Public Sub New()
            Me.HeadId = 0
            Me.LinkName = "Need to update this"
            Me.Description = "Need to update this"
        End Sub
        Public Sub New(ByVal pLinkName As String, ByVal pDescription As String)
            Me.LinkName = pLinkName
            Me.Description = pDescription
        End Sub
        Public Sub New(ByVal pHeadId As Int32, ByVal pLinkName As String, ByVal pDescription As String)
            Me.HeadId = pHeadId
            Me.LinkName = pLinkName
            Me.Description = pDescription
        End Sub
        Protected Sub New(ByVal reader As SqlDataReader)
            Me.HeadId = Convert.ToInt32(reader("HeadId"))
            Me.LinkName = reader("LinkName").ToString()
            Me.Description = reader("Description").ToString()
        End Sub
#End Region

#Region "Properties"

        ''' <summary>
        ''' Is used to set or get a HeadId regarding one headcategory.
        ''' </summary>
        Public Property HeadId() As Int32
            Get
                Return _HeadId
            End Get
            Set(ByVal value As Int32)
                _HeadId = value
            End Set
        End Property

        ''' <summary>
        ''' Is used to set or get a linkname regarding one headcategory.
        ''' </summary>
        Public Property LinkName() As String
            Get
                Return _LinkName
            End Get
            Set(ByVal value As String)
                _LinkName = value
            End Set
        End Property

        ''' <summary>
        ''' Is used to set or get a description regarding one headcategory.
        ''' </summary>
        Public Property Description() As String
            Get
                Return _Description
            End Get
            Set(ByVal value As String)
                _Description = value
            End Set
        End Property

#End Region

#Region "Methods"

        ''' <summary>
        ''' Insert one headcategory row with LinkName and Description based on the values for the 
        ''' variables LinkName and Description. You need to set values for these properties before calling
        ''' this method, HeadId is a identity column that increments with 1.
        ''' </summary>
        ''' <example>This example shows how to use this method with an empty constructor
        ''' <code>
        ''' Dim InsertRow As New HeadCategories()
        ''' InsertRow.LinkName = txtLinkName.Text
        ''' InsertRow.Description = txtDescription.Text
        ''' InsertRow.InsertOneRow()
        ''' </code>
        ''' </example>
        ''' <example2>This example shows how to use this method when parameters are supplied to the constructor
        ''' <code>
        ''' Dim InsertRow As New HeadCategories(txtLinkName.Text, txtDescription.Text)
        ''' InsertRow.InsertOneRow()
        ''' </code>
        ''' </example2>
        Public Overloads Sub InsertOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "INSERT INTO HeadCategories (LinkName, Description) VALUES (@LinkName, @Description)"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50).Value = Me.LinkName
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200).Value = Me.Description

                    cn.Open()
                    ' Execute the INSERT statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Insert one headcategory row by using the parameter values supplied when this method is called, 
        ''' the HeadId column is a identity that increments with 1.
        ''' </summary>
        ''' <param name="pLinkName">A short name for links in a menu (varchar, 50)</param>
        ''' <param name="pDescription">A 200 character description of the headcategory</param>
        ''' <example>This example shows how to use this method
        ''' <code>
        ''' Dim InsertRow As New HeadCategories()
        ''' InsertRow.InsertOneRow(txtLinkName.Text, txtDescription.Text)
        ''' </code>
        ''' </example>
        Public Overloads Sub InsertOneRow(ByVal pLinkName As String, ByVal pDescription As String)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "INSERT INTO HeadCategories (LinkName, Description) VALUES (@LinkName, @Description)"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50).Value = pLinkName
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200).Value = pDescription

                    cn.Open()
                    ' Execute the INSERT statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Insert many headcategory rows with a SqlDataAdapter and a DataTable, create a datatable that contains
        ''' two columns (LinkName and Description).
        ''' </summary>
        ''' <param name="pHeadCategoryDT">A DataTable with two columns (LinkName(50) and Desription(200))</param>
        '''<example>This example shows how to create a DataTable and to use this method
        '''<code>
        ''' // The Using block is used to automatically call dispose for the DataTable when it is not used any more
        ''' Using ManyRowsDataTable As New DataTable
        ''' Declare variables for DataColumn and DataRow objects.
        ''' Dim column As DataColumn
        ''' Dim row As DataRow
        ''' // Create and add the first column
        ''' column = New DataColumn("LinkName")
        ''' ManyRowsDataTable.Columns.Add(column)
        ''' // Create and add the second column
        ''' column = New DataColumn("Description")
        ''' ManyRowsDataTable.Columns.Add(column)
        ''' // Declare variables for a tablerow, rownumber and total number of table rows (ignoring the TableHeaderRow)
        ''' Dim CurrentRow As Int32
        ''' Dim RowNumber As Int32 = 1
        ''' Dim TotalOfTableRows As Int32 = ManyRowsTable.Rows.Count - 1
        ''' // A For loop to loop through each table row in our ASP.NET table and to add rows in our ManyRowsDataTable
        ''' For CurrentRow = 1 To TotalOfTableRows
        ''' //Find the textboxes in each table row
        ''' Dim LinkNameText As TextBox = ManyRowsTable.FindControl("txtLinkName" + RowNumber)
        ''' Dim DescriptionText As TextBox = ManyRowsTable.FindControl("txtDescription" + RowNumber)
        ''' // Add a row in the ManyRowsDataTable if the values in both of the textboxes not is blank
        ''' If Not LinkNameText.Text = String.Empty And Not DescriptionText.Text = String.Empty Then
        ''' // Add one row with values in the two columns LinkName and Description
        ''' row = ManyRowsDataTable.NewRow()
        ''' row("LinkName") = LinkNameText.Text
        ''' row("Description") = DescriptionText.Text
        ''' ManyRowsDataTable.Rows.Add(row)
        ''' // Add 1 to the rownumber
        ''' RowNumber = RowNumber + 1
        ''' End If
        ''' Next CurrentRow
        ''' // Declare InsertManyRows as a new instance of the HeadCategories class
        ''' Dim InsertManyRows As New HeadCategories()
        ''' // Call the method InsertManyRows in the HeadCategories class and add the DataTable as a parameter
        ''' InsertManyRows.InsertManyRows(ManyRowsDataTable)
        ''' End Using
        ''' </code>
        ''' </example>
        Public Sub InsertManyRows(ByVal pHeadCategoryDT As DataTable)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "INSERT INTO HeadCategories (LinkName, Description) VALUES (@LinkName, @Description)"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn), _
                      ad As New SqlDataAdapter()
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50, "LinkName")
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200, "Description")

                    cn.Open()
                    ' Set the insertcommand for the SqlDataAdapter
                    ad.InsertCommand = cmd
                    ' Excecute the INSERT statement
                    ad.Update(pHeadCategoryDT)
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

        End Sub

        ''' <summary>
        ''' Insert many headcategory rows with SqlBulkCopy and a DataTable, create a datatable that contains
        ''' two columns (LinkName and Description).
        ''' </summary>
        ''' <param name="pHeadCategoryDT">A DataTable contain two columns (LinkName(50) and Desription(200))</param>
        '''<example>This example shows how to create a DataTable and to use this method
        '''<code>
        ''' // The Using block is used to automatically call dispose for the DataTable when it is not used any more
        ''' Using ManyRowsDataTable As New DataTable
        ''' Declare variables for DataColumn and DataRow objects.
        ''' Dim column As DataColumn
        ''' Dim row As DataRow
        ''' // Create and add the first column
        ''' column = New DataColumn("LinkName")
        ''' ManyRowsDataTable.Columns.Add(column)
        ''' // Create and add the second column
        ''' column = New DataColumn("Description")
        ''' ManyRowsDataTable.Columns.Add(column)
        ''' // Declare variables for a tablerow, rownumber and total number of table rows (ignoring the TableHeaderRow)
        ''' Dim CurrentRow As Int32
        ''' Dim RowNumber As Int32 = 1
        ''' Dim TotalOfTableRows As Int32 = ManyRowsTable.Rows.Count - 1
        ''' // A For loop to loop through each table row in our ASP.NET table and to add rows in our ManyRowsDataTable
        ''' For CurrentRow = 1 To TotalOfTableRows
        ''' //Find the textboxes in each table row
        ''' Dim LinkNameText As TextBox = ManyRowsTable.FindControl("txtLinkName" + RowNumber)
        ''' Dim DescriptionText As TextBox = ManyRowsTable.FindControl("txtDescription" + RowNumber)
        ''' // Add a row in the ManyRowsDataTable if the values in both of the textboxes not is blank
        ''' If Not LinkNameText.Text = String.Empty And Not DescriptionText.Text = String.Empty Then
        ''' // Add one row with values in the two columns LinkName and Description
        ''' row = ManyRowsDataTable.NewRow()
        ''' row("LinkName") = LinkNameText.Text
        ''' row("Description") = DescriptionText.Text
        ''' ManyRowsDataTable.Rows.Add(row)
        ''' // Add 1 to the rownumber
        ''' RowNumber = RowNumber + 1
        ''' End If
        ''' Next CurrentRow
        ''' // Declare InsertBulk as a new instance of the HeadCategories class
        ''' Dim InsertBulk As New HeadCategories()
        ''' // Call the method InsertSqlBulk in the HeadCategories class and add the DataTable as a parameter
        ''' InsertBulk.InsertSqlBulk(ManyRowsDataTable)
        ''' End Using
        ''' </code>
        ''' </example>
        Public Sub InsertSqlBulk(ByVal pHeadCategoryDT As DataTable)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      bc As New SqlBulkCopy(cn)

                    cn.Open()

                    ' The HeadCategories table has 3 columns but the pHeadCategoryDT DataTable have only 
                    ' two columns and we therefore have to add column mappings to match the DataTable with
                    ' the MS SQL database table.
                    bc.ColumnMappings.Add("LinkName", "LinkName")
                    bc.ColumnMappings.Add("Description", "Description")

                    ' Set the name of the destination table in the database
                    bc.DestinationTableName = "HeadCategories"

                    ' Write from the DataTable to the destination table
                    bc.WriteToServer(pHeadCategoryDT)

                End Using

            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

        End Sub

        ''' <summary>
        ''' Update one headcategory row where the HeadId equals the HeadId variable in this class, you need to set values
        ''' for HeadId, LinkName and Description with a constructor or with properties before using this method.
        ''' </summary>
        ''' <example>This example shows how to use this method with an empty constructor
        ''' <code>
        ''' // Declare UpdateOneRow as a new instance of the HeadCategories class ignoring to supply parameters
        ''' Dim UpdateOneRow As New HeadCategories()
        ''' // Set the properties for HeadId, LinkName and Description
        ''' UpdateOneRow.HeadId = txtHeadId.Text
        ''' UpdateOneRow.LinkName = txtLinkName.Text
        ''' UpdateOneRow.Description = txtDescription.Text
        ''' // Call the method to update one row without any parameters
        ''' UpdateOneRow.UpdateOneRow()
        ''' </code>
        ''' </example>
        ''' <example2>This example shows how to use this method when parameters are supplied to the constructor
        ''' <code>
        ''' // Declare UpdateOneRow as a new instance of the HeadCategories class with parameters in the constructor
        ''' Dim UpdateOneRow As New HeadCategories(txtHeadId.Text, txtLinkName.Text, txtDescription.Text)
        ''' // Call the method to update one row without any parameters
        ''' UpdateOneRow.UpdateOneRow()
        ''' </code>
        ''' </example2>
        Public Overloads Sub UpdateOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "UPDATE HeadCategories SET LinkName = @LinkName, Description = @Description WHERE HeadId = @Id"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = Me.HeadId
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50).Value = Me.LinkName
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200).Value = Me.Description

                    cn.Open()
                    ' Excecute the UPDATE statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Update one headcategory row where the HeadId equals the supplied parameter pId, you need to add 
        ''' three parameters to use this method.
        ''' </summary>
        ''' <param name="pId">Id number for a headcategory</param>
        ''' <param name="pLinkName">A short name for links in a menu (varchar, 50)</param>
        ''' <param name="pDescription">A 200 character despription of the headcategory</param>
        ''' <example>This example shows how to use this method
        ''' <code>
        ''' // Declare UpdateOneRow as a new instance of the HeadCategories class without parameters in the constructor
        ''' Dim UpdateOneRow As New HeadCategories()
        ''' // Call the method to update one row and supply parameters to the method
        ''' UpdateOneRow.UpdateOneRow(txtHeadId.Text, txtLinkName.Text, txtDescription.Text)
        ''' </code>
        ''' </example>
        Public Overloads Sub UpdateOneRow(ByVal pId As Int32, ByVal pLinkName As String, ByVal pDescription As String)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "UPDATE HeadCategories SET LinkName = @LinkName, Description = @Description WHERE HeadId = @Id"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = pId
                    cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50).Value = pLinkName
                    cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200).Value = pDescription

                    cn.Open()
                    ' Excecute the UPDATE statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Select one headcategory row where the HeadId equals the value for the variable HeadId in this class, which you
        ''' have to set with a constructor or a property before using this method. The selected row is used to set the
        ''' properties for HeadId, LinkName and Description so you have to get values from the properties to get the
        ''' selected data.
        ''' </summary>
        ''' <example>This example shows how to use this method with an empty constructor
        ''' <code>
        ''' // Declare SelectOneRow as a new instance of the HeadCategories class ignoring to supply parameters
        ''' Dim SelectOneRow As New HeadCategories()
        ''' // Set the property for HeadId
        ''' SelectOneRow.HeadId = txtInputHeadId.Text
        ''' //  Call the method to select one row and ignore to supply parameters
        ''' SelectOneRow.SelectOneRow()
        ''' //  Get the properties for HeadId, LinkName and Description
        ''' txtHeadId.Text = SelectOneRow.HeadId
        ''' txtLinkName.Text = SelectOneRow.LinkName
        ''' txtDescription.Text = SelectOneRow.Description
        ''' </code>
        ''' </example>
        ''' <example2>This example shows how to use this method when parameters are supplied to the constructor
        ''' <code>
        ''' // Declare SelectOneRow as a new instance of the HeadCategories class and supply parameters in the constructor
        ''' Dim SelectOneRow As New HeadCategories(txtInputHeadId.Text, String.Empty, String.Empty)
        ''' // Call the method to select one row and ignore to supply parameters
        ''' SelectOneRow.SelectOneRow()
        ''' // Get the properties for HeadId, LinkName and Description
        ''' txtHeadId.Text = SelectOneRow.HeadId
        ''' txtLinkName.Text = SelectOneRow.LinkName
        ''' txtDescription.Text = SelectOneRow.Description
        ''' </code>
        ''' </example2>
        Public Overloads Sub SelectOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM HeadCategories WHERE HeadId = @HeadId"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@HeadId", SqlDbType.Int).Value = Me.HeadId

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            Me.HeadId = Convert.ToInt32(reader("HeadId"))
                            Me.LinkName = reader("LinkName").ToString()
                            Me.Description = reader("Description").ToString()
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Select one headcategory row where HeadId equals pID, you need supply a parameter value for HeadId to use
        ''' this method. The selected row is used to set values for the properties: HeadId, LinkName and Description.
        ''' You need to get the values from these properties after calling this method.
        ''' </summary>
        ''' <param name="pId">Id number for a headcategory</param>
        ''' <example>This example shows how to use this method
        ''' <code>
        ''' // Declare SelectOneRow as a new instance of the HeadCategories class without parameters in the constructor
        ''' Dim SelectOneRow As New HeadCategories()
        ''' // Call the method to select one row and supply a parameter representing HeadId
        ''' SelectOneRow.SelectOneRow(txtInputHeadId.Text)
        ''' // Get the properties for HeadId, LinkName and Description
        ''' txtHeadId.Text = SelectOneRow.HeadId
        ''' txtLinkName.Text = SelectOneRow.LinkName
        ''' txtDescription.Text = SelectOneRow.Description
        ''' </code>
        ''' </example>
        Public Overloads Sub SelectOneRow(ByVal pID As Int32)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM HeadCategories WHERE HeadId = @HeadId"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@HeadId", SqlDbType.Int).Value = pID

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            Me.HeadId = Convert.ToInt32(reader("HeadId"))
                            Me.LinkName = reader("LinkName").ToString()
                            Me.Description = reader("Description").ToString()
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Select one headcategory row where HeadId equals pId, you have to supply a value for the parameter pId to use
        ''' this method. The selected row is returned as an array of 3 strings (0 to 2).
        ''' </summary>
        ''' <param name="pId">Id number for a headcategory</param>
        ''' <returns>A one-dimensional 0-based array with 3 strings (HeadId, LinkName, Description)</returns>
        ''' <example>This example shows how to use this method
        ''' <code>
        ''' // Declare SelectOneRow as a new instance of the HeadCategories class without parameters in the constructor
        ''' Dim SelectOneRow As New HeadCategories()
        ''' // Declare a string array
        ''' Dim SingleRowArray() As String
        ''' // Call the array function to select one row and supply a parameter representing HeadId, the returned value
        ''' // are stored in the SingleRowArray.
        ''' SingleRowArray = SelectOneRow.SelectOneRowToArray(txtInputHeadId.Text)
        ''' // Get HeadId, LinkName and Description from the string array
        ''' txtHeadId.Text = SingleRowArray(0)
        ''' txtLinkName.Text = SingleRowArray(1)
        ''' txtDescription.Text = SingleRowArray(2)
        ''' </code>
        ''' </example>
        Public Function SelectOneRowToArray(ByVal pId As Int32) As String()

            ' Declare a OneRowArray as a StringArray with 3 values (0 to 2)
            Dim OneRowArray(2) As String

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM HeadCategories WHERE HeadId = @HeadId"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@HeadId", SqlDbType.Int).Value = pId

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            ' Fill the StringArray with values
                            OneRowArray(0) = reader("HeadId").ToString()
                            OneRowArray(1) = reader("LinkName").ToString()
                            OneRowArray(2) = reader("Description").ToString()
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a one-dimensional String array
            Return OneRowArray

        End Function

        ''' <summary>
        ''' Select one headcategory row where HeadId are equal to the supplied value for the parameter pId. This function
        ''' returns an array list with one headcategory object that is created with a protected constructor that accepts
        ''' a SqlDataReader as a parameter.
        ''' </summary>
        ''' <param name="pId">Id number for a headcategory</param>
        ''' <returns>A ArrayList with one headcategory object (HeadId, LinkName, Description)</returns>
        ''' <example>This example shows how to use this method
        ''' <code>
        ''' // Declare SelectOneRow as a new instance of the HeadCategories class without parameters in the constructor
        ''' Dim SelectOneRow As New HeadCategories()
        ''' // Declare a new instance of an ArrayList
        ''' Dim SingleRowArrayList As New ArrayList()
        ''' // Call the ArrayList function to select one row and supply a parameter representing HeadId, the returned value
        ''' // are stored in the SingleRowArrayList.
        ''' SingleRowArrayList = SelectOneRow.SelectOneRowToArrayList(txtInputHeadId.Text)
        ''' // Loop through the ArrayList of HeadCategories (objects) and get the value from each property, it is only one item
        ''' // in the ArrayList so the will only be one loop.
        ''' For Each SelectOneRow In SingleRowArrayList
        '''    txtHeadId.Text = SelectOneRow.HeadId
        '''    txtLinkName.Text = SelectOneRow.LinkName
        '''    txtDescription.Text = SelectOneRow.Description
        ''' Next
        ''' </code>
        ''' </example>
        Public Function SelectOneRowToArrayList(ByVal pId As Int32) As ArrayList

            ' Declare OneRowToArrayList as a new ArrayList
            Dim OneRowToArrayList As New ArrayList

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM HeadCategories WHERE HeadId = @HeadId"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@HeadId", SqlDbType.Int).Value = pId

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            ' Add a headcategory object to the ArrayList
                            OneRowToArrayList.Add(New HeadCategories(reader))
                        End While
                    End Using
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a ArrayList
            Return OneRowToArrayList

        End Function

        ''' <summary>
        ''' Select all headcategory rows to an 2-dimensional Array of strings.
        ''' </summary>
        ''' <returns>An 2-dimensional Array of strings with three columns (HeadId, LinkName, Description)</returns>
        ''' <example>This example shows how to use this method
        ''' <code>
        ''' // Declare SelectAllFrom2DArray as a new instance of the HeadCategories class
        ''' Dim SelectAllFrom2DArray As New HeadCategories()
        ''' // Call the method SelectAllTo2DArray in the HeadCategories class an copy the returned value to a
        ''' // 2-dimensional StringArray
        ''' Dim TwoDimensionalArray As String(,) = SelectAllFrom2DArray.SelectAllTo2DArray()
        ''' // Declare variables for rows and cells that will be used to add rows and cells to the ArrayTable
        ''' Dim tRow As TableRow
        ''' Dim tCell As TableCell
        ''' // Create a new TableHeaderRow in the ArrayTable
        ''' Dim hRow As New TableHeaderRow
        ''' ArrayTable.Rows.Add(hRow)
        ''' // Create and add the first cell to the TableHeaderRow
        ''' tCell = New TableCell
        ''' tCell.Text = "Id"
        ''' hRow.Cells.Add(tCell)
        ''' // Create and add the second cell to the TableHeaderRow
        ''' tCell = New TableCell
        ''' tCell.Text = "Linkname"
        ''' hRow.Cells.Add(tCell)
        ''' // Create and add the third cell to the TableHeaderRow
        ''' tCell = New TableCell
        ''' tCell.Text = "Description"
        ''' hRow.Cells.Add(tCell)
        ''' // Get the total number of rows in the 2D array minus 1 (a 0-based index)
        ''' Dim RowsInArray As Int32 = TwoDimensionalArray.GetLength(0) - 1
        ''' // Declare Row as an integer
        ''' Dim Row As Int32
        ''' // Loop through the rows in the TwoDimensionalArray, add rows to the ArrayTable and add cells in each
        ''' // row.
        ''' For Row = 0 To RowsInArray
        '''   // Create and add a new row
        '''   tRow = New TableRow
        '''   ArrayTable.Rows.Add(tRow)
        '''   // Create and add a new cell
        '''   tCell = New TableCell
        '''   tCell.Text = TwoDimensionalArray(Row, 0)
        '''   tRow.Cells.Add(tCell)
        '''   // Create and add a new cell
        '''   tCell = New TableCell
        '''   tCell.Text = TwoDimensionalArray(Row, 1)
        '''   tRow.Cells.Add(tCell)
        '''   // Create and add a new cell
        '''   tCell = New TableCell
        '''   tCell.Text = TwoDimensionalArray(Row, 2)
        '''   tRow.Cells.Add(tCell)
        '''Next Row
        ''' </code>
        ''' </example>
        Public Function SelectAllTo2DArray() As String(,)

            ' Declare RowNumber as a Integer that will be used to fill the 2D array with values and TotalNumberOfRows
            ' as a Integer to be able to determine the size of the 2D array.
            Dim RowNumber As Int32 = 0
            Dim TotalNumberOfRows As Int32

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT Count(HeadId) As Rows FROM HeadCategories"
                Dim sql2 As String = "SELECT * FROM HeadCategories"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn), _
                      cmd2 As New SqlCommand(sql2, cn)

                    cn.Open()

                    ' Get the number of rows - 1
                    TotalNumberOfRows = Convert.ToInt32(cmd.ExecuteScalar()) - 1

                    ' Declare AllTo2DArray as a StringArray with rows and columns
                    Dim AllTo2DArray(TotalNumberOfRows, 2) As String

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd2.ExecuteReader()
                        While reader.Read()
                            ' Fill the 2D StringArray with values
                            AllTo2DArray(RowNumber, 0) = reader("HeadId").ToString()
                            AllTo2DArray(RowNumber, 1) = reader("LinkName").ToString()
                            AllTo2DArray(RowNumber, 2) = reader("Description").ToString()

                            ' Add 1 to RowNumber
                            RowNumber = RowNumber + 1
                        End While
                    End Using

                    ' Copy the the AllTo2DArray to the SelectAllTo2DArray
                    SelectAllTo2DArray = AllTo2DArray

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a 2-dimensional StringArray
            Return SelectAllTo2DArray

        End Function

        ''' <summary>
        ''' Select all headcategory rows to an ArrayList of headcategory objects (HeadId, LinkName, Description). An 
        ''' ArrayList of objects can be used as a datasource for a repeater control.
        ''' </summary>
        ''' <returns>An ArrayList of headcategory objects, each object consists of (HeadId, LinkName, Description)</returns>
        ''' <example>This example shows how to use this method
        ''' <code>
        ''' // Declare SelectAll as a new instance of the HeadCategories class without any parameters
        ''' Dim SelectAll As New HeadCategories()
        ''' // Set the datasource for our Repeater control to the ArrayList that are returned from the
        ''' // SelectAllToArrayList function in the HeadCategories class (no parameters)
        ''' SelectAllRepeater.DataSource = SelectAll.SelectAllToArrayList()
        ''' // Bind the datasource to the SelectAllRepeater 
        ''' SelectAllRepeater.DataBind()
        ''' </code>
        ''' </example>
        Public Function SelectAllToArrayList() As ArrayList

            ' Declare AllToArrayList as a new ArrayList
            Dim AllToArrayList As New ArrayList

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM HeadCategories"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader()
                        While reader.Read()
                            ' Add a headcategory object to the ArrayList
                            AllToArrayList.Add(New HeadCategories(reader))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a ArrayList
            Return AllToArrayList

        End Function

        ''' <summary>
        ''' Select all headcategory rows to a DataTable with three columns (HeadId, LinkName, Description). A DataTable
        ''' can be used as a datasource for a repeater control.
        ''' </summary>
        ''' <returns>A DataTable three columns (HeadId, LinkName, Description) that contains headcategory rows</returns>
        ''' <example>This example shows how to use this method
        ''' <code>
        ''' // Declare SelectAll as a new instance of the HeadCategories class without any parameters
        ''' Dim SelectAll As New HeadCategories()
        ''' // Set the datasource for our Repeater control to the DataTable that are returned from the
        ''' // SelectAllToDataTable function in the HeadCategories class (no parameters)
        ''' SelectAllRepeater.DataSource = SelectAll.SelectAllToDataTable()
        ''' // Bind the datasource to the SelectAllRepeater 
        ''' SelectAllRepeater.DataBind()
        ''' </code>
        ''' </example>
        Public Function SelectAllToDataTable() As DataTable

            ' Declare AllHeadCategories as a new DataTable
            Dim AllHeadCategories As New DataTable

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM HeadCategories"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn), _
                      ad As New SqlDataAdapter()

                    cn.Open()
                    ' Set the selectcommand for the SqlDataAdapter
                    ad.SelectCommand = cmd
                    ' Use the SqlDataAdapter to insert data in a DataTable
                    ad.Fill(AllHeadCategories)

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a DataTable
            Return AllHeadCategories

        End Function
#End Region

    End Class

End Namespace

